# /packages/photo-album/tcl/photo-album-procs.tcl

ad_library {
    TCL library for the photo-album system

    @author Tom Baginski (bags@arsdigita.com)
    @creation-date December 14, 2000
    @cvs-id $Id: photo-album-procs.tcl,v 1.1 2002/07/09 17:35:10 rmello Exp $
}

# wtem@olywa.net, 2001-09-19
# there are several procs that we may be able to replace 
# with standard cr procs now that it handles file system storage of images
# look for ### maybe redundant, replace calls?
# in front of said procs
# places to make partial changes are marked with
### probable partial change
# in front of them
# areas that have been worked on are marked
### clean
# changes that have been deferred marked
### skipped

ad_proc -public pa_get_root_folder {
    {package_id ""}
} {
    Returns the folder_id of the root folder for an instance of the photo album system.
    If no root folder exists, as when a new package instance is accessed for the first time,
    a new root folder is created automatically with appropriate permissions
    If value has be previously requested, value pulled from cache
} { 
    if [empty_string_p $package_id] {
	set package_id [ad_conn package_id]
    }
    return [util_memoize "pa_get_root_folder_internal $package_id"]
}

ad_proc -private pa_get_root_folder_internal {
    {package_id}
} {
    Returns the folder_id of the root folder for an instance of the photo album system.
    If no root folder exists, as when a new package instance is accessed for the first time,
    a new root folder is created automatically with appropriate permissions
} {
    set folder_id [db_string pa_root_folder "select photo_album.get_root_folder(:package_id) from dual"]

    if [empty_string_p $folder_id] {
	set folder_id [pa_new_root_folder $package_id]
    }
    return $folder_id
}

ad_proc -private pa_new_root_folder {
    {package_id ""}
} {
    Creates a new root folder for a package, and returns id.
    A hackish function to get around the fact that we can't run
    code automatically when a new package instance is created.

} {
    if [empty_string_p $package_id] {
	set package_id [ad_conn package_id]
    }

    # wtem@olywa.net, 2001-09-22
    # the PhotoDir parameter goes away with new CR storage scheme

    # wtem@olywa.net, 2001-09-22
    # original pl/sql wrapped up in function to simplify code and porting
    db_transaction {
	# create new root folder
    
	set new_folder_id [db_exec_plsql make_new_root {
	begin
	:1 := photo_album.new_root_folder(:package_id);
	end;
	}]
    
	# grant default permissions to new root folder

	# default permissions stored in package parmeter as a list of grantee privilege pairs
	# the grantee can be anything that is or returns a party_id such as an integer, a subquery,
	# or a function
	
	set perm_lst [split [ad_parameter DefaultRootFolderPrivileges] " "]

	foreach {party privilege}  $perm_lst {
	    # wtem@olywa.net, 2001-10-15
	    # urgh, originally the parameter had the pl/sql embedded in it
	    set grantee_id [db_string get_grantee_id "select acs.magic_object_id('$party') from dual"]
	  
	    db_exec_plsql grant_default {
		begin
		  acs_permission.grant_permission (
	            object_id  => :new_folder_id,
	            grantee_id => :grantee_id,
	            privilege  => :privilege
		  );
		end;
	    }
	}
    }
    # since this is executed the first time a package instance is accessed,
    # make sure the upload dir exists

    ### probable partial change
    # wtem@olywa.net, 2001-09-19
    # this initializes corresponding directory structure in the file-system
    # we should be able to eliminate this now that we are using CR storage
    # pa_assert_dir -check_base_path ""
    ### clean
    return $new_folder_id
}

ad_proc -public pa_get_folder_name {
    folder_id
} {
    Returns the name of a folder. 
} {
    return [db_string folder_name "
    select content_folder.get_label(:folder_id) from dual"]
}

ad_proc -public pa_get_folder_description {
    folder_id
} {
    Returns the description of a folder. 
} {
    return [db_string folder_description "
    select description from cr_folders where folder_id = :folder_id"]
}


ad_proc pa_context_bar_list {
    {-final ""}
    item_id
} {
    Constructs the list to be fed to ad_context_bar appropriate for
    item_id.  If -final is specified, that string will be the last 
    item in the context bar.  Otherwise, the name corresponding to 
    item_id will be used.

    modified from fs_context_bar
} {
    set root_folder_id [pa_get_root_folder]

    if {$item_id == $root_folder_id} {
	return
    }

    if [empty_string_p $final] {
	# set start_id and final with a single trip to the database

	db_1row get_start_and_final "select parent_id as start_id,
	  content_item.get_title(item_id,'t') as final
	  from cr_items where item_id = :item_id"

	#set start_id [db_string parent_id "
	#select parent_id from cr_items where item_id = :item_id"]
	#set final [db_string title "select content_item.get_title(:item_id,'t') from dual"]

    } else {
	set start_id $item_id
    }

    if {$start_id != $root_folder_id} {
    set context_bar [db_list_of_lists context_bar "
    select decode(
             content_item.get_content_type(i.item_id),
             'content_folder',
             'index?folder_id=',
             'pa_album',             
             'album?album_id=',
             'photo?photo_id='
           ) || i.item_id,
           content_item.get_title(i.item_id,'t')
    from   cr_items i
    connect by prior i.parent_id = i.item_id
      and i.item_id != :root_folder_id
    start with item_id = :start_id
    order by level desc"]
    }
    lappend context_bar $final

    return $context_bar
}

# wtem@olywa.net, 2001-09-24
### clean
ad_proc -public pa_make_file_name {
    {-ext ""}
    id
} { 
    constructs a filename for an image based on id and extention.
} {
    if {![empty_string_p $ext] && ![regexp {^\.} $ext foo]} {
	#add back the dot
	set ext ".${ext}"
    }
    set total_file_name "${id}${ext}"

    return $total_file_name
}

### maybe redundant, replace calls?
# wtem@olywa.net, 2001-09-22
### skipped
# will handle when we get to the places where this is called
ad_proc -private pa_assert_dir {
    {-check_base_path:boolean}
    {dir_path}
} {
    Ensures that dirname exists under the PhotoDir Directory  
    If -check_base_path flag specified, proc also checks if base path exists and adds it if necessary

    Won't cause an error if the directory is already there. Better than the stardard
    mkdir because it will make all the directories leading up to dirname
    borrowed from 3.4 download code
} {  
    ns_log Debug "Checking dir, pa_assert_dir"
    if { $check_base_path_p } {
	set dir_path "[acs_root_dir]/[ad_parameter PhotoDir]/$dir_path"
	set needed_dir ""
    } else {
	set needed_dir "[acs_root_dir]/[ad_parameter PhotoDir]"
    }
    ns_log Debug "dir_path: $dir_path \n needed_dir: $needed_dir"
    
    set dir_list [split $dir_path /]
    
    foreach dir $dir_list {
	ns_log Debug "Checking: $dir"
        if [empty_string_p $dir] {
            continue
        }
        append needed_dir "/$dir"
        if ![file exists $needed_dir] {
	    ns_log Debug "Making: $dir"
            ns_mkdir $needed_dir
        }
    }
}

ad_proc -public pa_is_folder_p {
    folder_id
    {package_id ""}
} {
    returns "t" if folder_id is a folder that is a child of the root folder for the package,
    else "f"
}  {
    return [pa_is_type_in_package $folder_id "content_folder" $package_id]
}

ad_proc -public pa_is_album_p {
    album_id
    {package_id ""}
} {
    returns "t" if album_id is a pa_album that is a child of the root folder for the package,
    else "f"
}  {
    return [pa_is_type_in_package $album_id "pa_album" $package_id]
}

ad_proc -public pa_is_photo_p {
    photo_id
    {package_id ""}
} {
    returns "t" if photo_id is a pa_photo that is a child of the root folder for the package,
    else "f"
}  {
    return [pa_is_type_in_package $photo_id "pa_photo" $package_id]
}

ad_proc -private pa_is_type_in_package {
    item_id
    content_type
    {package_id ""}
} {
    returns "t" if item_id is of the specified content_type and is a child of the root folder of the package,
    else returns "f"
} {
    set root_folder [pa_get_root_folder $package_id]

    # I check for the case that item is the root_folder first because this happens on the index page.
    # Since index page accessed often, and the root_folder is within the package this avoids an unecessary
    # trip to the database on a commonly accessed page.
 
    if {[string equal $content_type "content_folder"] && [string equal $item_id $root_folder]} {
	return "t"
    } else {
	return [db_string check_is_type_in_package "select decode((select 1 
	from dual
	where exists (select 1 
	    from cr_items 
	    where item_id = :root_folder 
	    connect by prior parent_id = item_id 
	    start with item_id = :item_id)
	  and content_item.get_content_type(:item_id) = :content_type
	), 1, 't', 'f')
	from dual" ]
    }
}

ad_proc -public pa_grant_privilege_to_creator {
    object_id
    {user_id ""}
} {
    Grants a set of default privileges stored in parameter PrivilegeForCreator
    on object id to user_id.  If user_id is not specified, uses current user.
} {
    if {[empty_string_p $user_id]} {
	set user_id [ad_conn user_id]
    }
    set grant_list [split [ad_parameter PrivilegeForCreator] ","]
    foreach privilege $grant_list {
	db_exec_plsql grant_privilege {
	    begin
	      acs_permission.grant_permission (
	        object_id  => :object_id,
	        grantee_id => :user_id,
	        privilege  => :privilege
	      );
	    end;
	}
    } 
}

ad_proc -public pa_image_width_height {
    filename
    width_var
    height_var
} {
    Uses ImageMagick program to get the width and height in pixels of filename.
    Sets height to the variable named in height_var in the calling level.
    Sets width_var to the variable named in width_var in the calling level.

    I Use ImageMagick instead of aolsever funcition because it can handle more than
    just gifs and jpegs.  
} {
    set identify_string [exec [ad_parameter ImageMagickPath]/identify $filename]
    regexp {[ ]+([0-9]+)[x]([0-9]+)[\+]*} $identify_string x width height
    uplevel "set $width_var $width"
    uplevel "set $height_var $height"
}

ad_proc -public pa_make_new_image {
    base_image
    new_image
    max_width
    {max_height ""}
} {
    Uses ImageMagick program to create a file named new_image from base_image that 
    fits within a box defined by max_width by max_height (in pixels).
    ImageMagick will retain the aspect ratio of the base_image when creating the new_image
    If max_height is not specified, max_width is used for max_height.
} {
    if {[empty_string_p $max_height]} {
	set max_height $max_width
    }
    ns_log debug "pa_make_new_image: Start convert, making $new_image"
    exec [ad_parameter ImageMagickPath]/convert -interlace NONE -geometry ${max_width}x${max_height} $base_image $new_image
    ns_log debug "pa_make_new_image: Done convert."
}

# wtem@olywa.net, 2001-09-22
# replaced pa_delete_scheduled_files with standard cr_delete_scheduled_files


ad_proc -public pa_all_photos_in_album {
    album_id
} {
    returns a list of all the photo_ids in an album sorted in ascending order
    pull value from cache if already there, caches result and returns result if not
} {
    return [util_memoize "pa_all_photos_in_album_internal $album_id"]
}

ad_proc -private pa_all_photos_in_album_internal {
    album_id
} {
    queries and returns a list of all photo_ids in album_id in ascending order 
} {
    return [db_list get_photo_ids "select 
  ci.item_id
from cr_items ci,
  cr_child_rels ccr
where ci.live_revision is not null
  and ci.content_type = 'pa_photo'
  and ccr.parent_id = :album_id
  and ci.item_id = ccr.child_id
order by ccr.order_n"]
}

ad_proc -public pa_count_photos_in_album {
    album_id
} {
    returns count of number of photos in album_id
} {  
    return [llength [pa_all_photos_in_album $album_id]]
}

ad_proc -public pa_all_photos_on_page {
    album_id 
    page
} {
    returns a list of the photo_ids on page page of album_id
    list is in ascending order
} {
    set images_per_page [ad_parameter ThumbnailsPerPage]
    set start_index [expr $images_per_page * ($page-1)]
    set end_index [expr $start_index + ($images_per_page - 1)]
    return [lrange [pa_all_photos_in_album $album_id] $start_index $end_index]
}

ad_proc -public pa_count_pages_in_album {
    album_id
} {
    returns the number of pages in album_id
} {
    return [expr int(ceil([pa_count_photos_in_album $album_id] / [ad_parameter ThumbnailsPerPage].0))]
}

ad_proc -public pa_page_of_photo_in_album {
    photo_id
    album_id
} {
    returns the page number of a photo in an album
    If photo is not in the album returns -1
} {
    set photo_index [lsearch [pa_all_photos_in_album $album_id] $photo_id]

    if {$photo_index == -1 } {
	return -1
    }

    return [expr int(ceil(($photo_index + 1)/ [ad_parameter ThumbnailsPerPage].0))]
}

ad_proc -public pa_flush_photo_in_album_cache {
    album_id
} {
    Clears the cacheed value set by pa_all_photos_in_album for a single album
    Call proc on any page that alters the number or order of photos in an album.
} {
    util_memoize_flush "pa_all_photos_in_album_internal $album_id"
}

# pagination procs based on cms pagintion-procs.tcl
# when these procs get rolled into acs distribution, should use them directly

ad_proc -deprecated pa_pagination_paginate_query { 
    sql
    page
} {
    takes a query and returns a query that accounts for pagination
} {

    set rows_per_page [ad_parameter ThumbnailsPerPage]
    set start_row [expr $rows_per_page*[expr $page-1]+1]

    set query "
      select *
      from
        (
          select 
            x.*, rownum as row_id
	  from
	    ($sql) x
        ) ordered_sql_query_with_row_id
      where
        row_id between $start_row and $start_row + $rows_per_page - 1"
    
    return $query
}

ad_proc -deprecated pa_pagination_get_total_pages {} {
    returns the total pages in a datasource defined by $sql
    The sql var must be defined at the calling level.
    Uplevel used so that any binde vars in query are defined
} {
    uplevel {
	return [db_string get_total_pages "
	select 
	ceil(count(*) / [ad_parameter ThumbnailsPerPage])
	from
	($sql)
	"]
    }
}

ad_proc -public pa_pagination_page_number_links { 
    page
    total_pages 
} {
    html for navigating pages of a a datasource
} {

    if { $total_pages == 1 } {
	return ""
    }

    set url [ns_conn url]
    set page_vars [ns_getform]

    set rows_per_page [ad_parameter ThumbnailsPerPage]

    set pagination_html ""



    # append the 'prev' link
    append pagination_html "
      <table border=0 cellspacing=0 cellpadding=4 width=95%>
      <tr bgcolor=\"#ffffff\"><td align=left width=10%>"
    if { $page > 1 } {
	ns_set update $page_vars page [expr $page-1]
	set url_vars [pa_pagination_ns_set_to_url_vars $page_vars]

	append pagination_html "
	  <a href=\"$url?$url_vars\">&lt;&lt; Prev </a>"
    } else {
	append pagination_html "&nbsp;"
    }
    append pagination_html "</td><td align=center width=80%>"

    # append page number links for all pages except for this page
    for { set i 1 } { $i <= $total_pages } { incr i } {
	if { $i == $page } {
	    append pagination_html " $page "
	} else {
	    ns_set update $page_vars page $i
	    set url_vars [pa_pagination_ns_set_to_url_vars $page_vars]
	    append pagination_html "
	      <a href=\"$url?$url_vars\">$i</a>"
	}
    }
    append pagination_html "</td><td align=right width=10%>"

    # append the 'next' link
    if { $page < $total_pages } {
	ns_set update $page_vars page [expr $page+1]
	set url_vars [pa_pagination_ns_set_to_url_vars $page_vars]

	append pagination_html "
	  <a href=\"$url?$url_vars\"> Next &gt;&gt;</a>"
    } else {
	append pagination_html "&nbsp;"
    }
    append pagination_html "</td></tr></table>"

    return $pagination_html
}

ad_proc -private pa_pagination_ns_set_to_url_vars { 
    set_id
} {
    helper procedure - turns an ns_set into a list of url_vars
} {
    set url_vars ""
    set size [ns_set size $set_id]
    for { set i 0 } { $i < $size } { incr i } {
	set key [ns_set key $set_id $i]
	set value [ns_set get $set_id $key]
	append url_vars "$key=$value&"
    }
    set url_vars [string trimright $url_vars "&"]
    return $url_vars
}

ad_proc -public pa_photo_number_links { 
    cur_id
    all_ids
} {
    given a current photo_id and and an ordered list of all the photo_id in an album
    creates an html fragment that allows user to navigate to any photo by number 
    next/previous
} {
    set location [lindex [ad_conn urlv] [expr [ad_conn urlc] -1]]

    if { [empty_string_p $cur_id] || [llength $all_ids] == 0 } {
	return ""
    }

    set cur_index [lsearch -exact $all_ids $cur_id]
    set prev_id [lindex $all_ids [expr $cur_index - 1]]
    set next_id [lindex $all_ids [expr $cur_index + 1]]
    set photo_nav_html ""

    # append the 'prev' link
    append photo_nav_html "
      <table border=0 cellspacing=0 cellpadding=4 width=95%>
      <tr bgcolor=\"#ffffff\"><td align=left width=10%>"
    if { ![empty_string_p $prev_id] } {
	append photo_nav_html "
	  <a href=\"${location}?photo_id=$prev_id\">&lt;&lt; Prev </a>"
    } else {
	append photo_nav_html "&nbsp;"
    }
    append photo_nav_html "</td><td align=center width=80%>"

    # append page number links for all pages except for this page
    set i 0
    foreach id $all_ids {
	incr i
	if { $cur_id == $id } {
	    append photo_nav_html " $i "
	} else {
	    append photo_nav_html "
	      <a href=\"${location}?photo_id=$id\"> $i </a>"
	}
	
    }
    append photo_nav_html "</td><td align=right width=10%>"

    # append the 'next' link
    if { ![empty_string_p $next_id] } {
	append photo_nav_html "
	  <a href=\"${location}?photo_id=$next_id\"> Next &gt;&gt;</a>"
    } else {
	append photo_nav_html "&nbsp;"
    }
    append photo_nav_html "</td></tr></table>"

    return $photo_nav_html
}
